@@ -308,7 +308,7 @@ class Star(Shape):
         r2 = self.outer_radius.get_value(time)
         rot = -(self.rotation.get_value(time)) * math.pi / 180 + math.pi
         p = self.points.get_value(time)
-        halfd = math.pi / p
+        halfd = -math.pi / p
 
         for i in range(int(p)):
             main_angle = rot + i * halfd * 2
@@ -417,7 +417,7 @@ class Path(Shape):
         return bb
 
     def to_bezier(self):
-        return self
+        return self.clone()
 
 
 ## \ingroup Lottie
@@ -490,6 +490,12 @@ class Group(ShapeElement):
         return results
 
 
+## \ingroup Lottie
+class FillRule(TgsEnum):
+    NonZero = 1
+    EvenOdd = 2
+
+
 ## \ingroup Lottie
 class Fill(ShapeElement):
     """!
@@ -498,7 +504,7 @@ class Fill(ShapeElement):
     _props = [
         TgsProp("opacity", "o", Value, False),
         TgsProp("color", "c", MultiDimensional, False),
-        TgsProp("r", "r", int, False),
+        TgsProp("fill_rule", "r", FillRule, False),
     ]
     ## %Shape type.
     type = "fl"
@@ -509,8 +515,8 @@ class Fill(ShapeElement):
         self.opacity = Value(100)
         ## Fill Color
         self.color = MultiDimensional(color or NVector(1, 1, 1))
-        ## \todo figure out?
-        self.r = None
+        ## Fill rule
+        self.fill_rule = None
 
 
 ## \ingroup Lottie
@@ -703,7 +709,13 @@ class Modifier(ShapeElement):
 
 
 ## \ingroup Lottie
-## \todo Implement SVG/SIF Export
+class TrimMultipleShapes(TgsEnum):
+    Simultaneously = 1
+    Individually = 2
+
+
+## \ingroup Lottie
+## \todo Implement SIF Export
 class Trim(Modifier):
     """
     Trims shapes into a segment
@@ -711,8 +723,8 @@ class Trim(Modifier):
     _props = [
         TgsProp("start", "s", Value, False),
         TgsProp("end", "e", Value, False),
-        TgsProp("angle", "o", Value, False),
-        TgsProp("m", "m", int, False),
+        TgsProp("offset", "o", Value, False),
+        TgsProp("multiple", "m", TrimMultipleShapes, False),
     ]
     ## %Shape type.
     type = "tm"
@@ -723,10 +735,10 @@ class Trim(Modifier):
         self.start = Value(0)
         ## End of the segment, as a percentage
         self.end = Value(100)
-        ## Angle where to start
-        self.angle = Value(0)
+        ## start/end offset, as an angle (0, 360)
+        self.offset = Value(0)
         ## \todo?
-        self.m = None
+        self.multiple = None
 
 
 ## \ingroup Lottie
