@@ -6,6 +6,7 @@ from .handler import SvgHandler, NameMode
 from ... import objects
 from ...nvector import NVector
 from ...utils import restructure
+from ...utils.transform import TransformMatrix
 try:
     from ...utils import font
     has_font = True
@@ -59,9 +60,11 @@ class SvgBuilder(SvgHandler, restructure.AbstractBuilder):
         return time
 
     def gen_id(self, prefix="id"):
-        #TODO should check if id_n already exists
-        self.idc += 1
-        id = "%s_%s" % (prefix, self.idc)
+        while True:
+            self.idc += 1
+            id = "%s_%s" % (prefix, self.idc)
+            if id not in self.ids:
+                break
         self.ids.add(id)
         return id
 
@@ -164,17 +167,43 @@ class SvgBuilder(SvgHandler, restructure.AbstractBuilder):
         elif isinstance(lot, objects.ImageLayer):
             use = ElementTree.SubElement(g, "use")
             use.attrib[self.qualified("xlink", "href")] = "#" + self._assets[lot.image_id]
+        elif isinstance(lot, objects.TextLayer):
+            self._on_text_layer(g, lot)
+        elif isinstance(lot, objects.SolidColorLayer):
+            rect = ElementTree.SubElement(g, "rect")
+            rect.attrib["width"] = str(lot.width)
+            rect.attrib["height"] = str(lot.height)
+            rect.attrib["fill"] = lot.color
 
         if not lot.name:
             g.attrib[self.qualified("inkscape", "label")] = lot.__class__.__name__
         if layer_builder.shapegroup:
             g.attrib["style"] = self.group_to_style(layer_builder.shapegroup)
+            self._split_stroke(layer_builder.shapegroup, g, dom_parent)
         if lot.hidden:
             g.attrib.setdefault("style", "")
             g.attrib["style"] += "display: none;"
 
         return g
 
+    def _on_text_layer(self, g, lot):
+        text = ElementTree.SubElement(g, "text")
+        doc = lot.data.get_value(self.time)
+        if doc:
+            text.attrib["font-family"] = doc.font_family
+            text.attrib["font-size"] = str(doc.font_size)
+            if doc.line_height:
+                text.attrib["line-height"] = "%s%%" % doc.line_height
+            if doc.justify == objects.text.TextJustify.Left:
+                text.attrib["text-align"] = "start"
+            elif doc.justify == objects.text.TextJustify.Center:
+                text.attrib["text-align"] = "center"
+            elif doc.justify == objects.text.TextJustify.Right:
+                text.attrib["text-align"] = "end"
+
+            text.attrib["fill"] = color_to_css(doc.color)
+            text.text = doc.text
+
     def _on_layer_end(self, out_layer):
         self._current_layer.pop()
 
@@ -204,47 +233,37 @@ class SvgBuilder(SvgHandler, restructure.AbstractBuilder):
 
     def set_transform(self, dom, transform):
         trans = []
-        pos = self._get_value(transform.position)
+        mat = TransformMatrix()
+
         anchor = self._get_value(transform.anchor_point)
-        pos -= anchor
-        if pos[0] != 0 or pos[1] != 0:
-            trans.append("translate(%s, %s)" % (pos.components[0], pos.components[1]))
+        mat.translate(-anchor.x, -anchor.y)
 
         scale = self._get_value(transform.scale, NVector(100, 100))
-        if scale[0] != 100 or scale[1] != 100:
-            scale /= 100
-            trans.append("scale(%s, %s)" % (scale.components[0], scale.components[1]))
+        mat.scale(scale.x / 100, scale.y / 100)
 
-        rot = self._get_value(transform.rotation, 0)
-        if rot != 0:
-            trans.append("rotate(%s, %s, %s)" % (rot, anchor[0], anchor[1]))
+        skew = self._get_value(transform.skew, 0) * math.pi / 180
+        if skew != 0:
+            axis = self._get_value(transform.skew_axis, 0) * math.pi / 180
+            mat.skew_from_axis(-skew, axis)
+
+        rot = self._get_value(transform.rotation, 0) * math.pi / 180
+        if rot:
+            mat.rotate(-rot)
+
+        # TODO auto orient
+
+        pos = self._get_value(transform.position)
+        mat.translate(pos.x, pos.y)
+
+        dom.attrib["transform"] = mat.to_css_2d()
 
         if transform.opacity is not None:
             op = transform.opacity.get_value(self.time)
             if op != 100:
                 dom.attrib["opacity"] = str(op/100)
 
-        skew = self._get_value(transform.skew, 0)
-        if skew != 0:
-            axis = self._get_value(transform.skew_axis, 0) * math.pi / 180
-            skx = skew * math.cos(axis)
-            sky = skew * math.sin(axis)
-            # TODO looks like skew moves things around in svg
-            trans.append("skewX(%s)" % skx)
-            trans.append("skewY(%s)" % sky)
-
-        if trans:
-            dom.attrib["transform"] = " ".join(trans)
-
-    def group_to_style(self, group):
+    def _get_group_stroke(self, group):
         style = {}
-        if group.fill:
-            style["fill-opacity"] = group.fill.opacity.get_value(self.time) / 100
-            if isinstance(group.fill, objects.GradientFill):
-                style["fill"] = "url(#%s)" % self.process_gradient(group.fill)
-            else:
-                style["fill"] = color_to_css(group.fill.color.get_value(self.time))
-
         if group.stroke:
             if isinstance(group.stroke, objects.GradientStroke):
                 style["stroke"] = "url(#%s)" % self.process_gradient(group.stroke)
@@ -283,12 +302,61 @@ class SvgBuilder(SvgHandler, restructure.AbstractBuilder):
                             last = 0
                         last_mode = dash.type
                 style["stroke-dasharray"] = " ".join(dasharray)
+        return style
 
+    def _style_to_css(self, style):
         return ";".join(map(
             lambda x: ":".join(map(str, x)),
             style.items()
         ))
 
+    def _split_stroke(self, group, fill_layer, out_parent):
+        if not group.stroke:# or group.stroke_above:
+            return
+
+        style = self._get_group_stroke(group)
+        if style.get("stroke-width", 0) <= 0 or style["stroke-opacity"] <= 0:
+            return
+
+        g = ElementTree.Element("g")
+        self.set_clean_id(g, "stroke")
+        use = ElementTree.Element("use")
+        for i, e in enumerate(out_parent):
+            if e is fill_layer:
+                out_parent.insert(i, g)
+                out_parent.remove(fill_layer)
+                break
+        else:
+            return
+
+        if group.stroke_above:
+            g.append(fill_layer)
+            g.append(use)
+        else:
+            g.append(use)
+            g.append(fill_layer)
+
+        use.attrib[self.qualified("xlink", "href")] = "#" + fill_layer.attrib["id"]
+        use.attrib["style"] = self._style_to_css(style)
+        return g
+
+    def group_to_style(self, group):
+        style = {}
+        if group.fill:
+            style["fill-opacity"] = group.fill.opacity.get_value(self.time) / 100
+            if isinstance(group.fill, objects.GradientFill):
+                style["fill"] = "url(#%s)" % self.process_gradient(group.fill)
+            else:
+                style["fill"] = color_to_css(group.fill.color.get_value(self.time))
+
+            if group.fill.fill_rule:
+                style["fill-rule"] = "evenodd" if group.fill.fill_rule == objects.FillRule.EvenOdd else "nonzero"
+
+        #if group.stroke_above:
+            #style.update(self._get_group_stroke(group))
+
+        return self._style_to_css(style)
+
     def process_gradient(self, gradient):
         spos = gradient.start_point.get_value(self.time)
         epos = gradient.end_point.get_value(self.time)
@@ -338,17 +406,18 @@ class SvgBuilder(SvgHandler, restructure.AbstractBuilder):
             self.set_id(path, group.paths.paths[0], force=True)
             path.attrib["style"] = self.group_to_style(group)
             self.set_transform(path, group.lottie.transform)
-            return
+            return self._split_stroke(group, path, dom_parent)
 
         g = self.group_from_lottie(group.lottie, dom_parent, group.layer)
         g.attrib["style"] = self.group_to_style(group)
         self.shapegroup_process_children(group, g)
-        return g
+        return self._split_stroke(group, g, dom_parent)
 
     def _on_merged_path(self, shape, shapegroup, out_parent):
         path = self.build_path(shape.paths, out_parent)
         self.set_id(path, shape.paths[0])
         path.attrib["style"] = self.group_to_style(shapegroup)
+        #self._split_stroke(shapegroup, path, out_parent)
         return path
 
     def _on_shape(self, shape, shapegroup, out_parent):
@@ -368,6 +437,8 @@ class SvgBuilder(SvgHandler, restructure.AbstractBuilder):
         if "style" not in svgshape.attrib:
             svgshape.attrib["style"] = ""
         svgshape.attrib["style"] += self.group_to_style(shapegroup)
+        #self._split_stroke(shapegroup, svgshape, out_parent)
+
         if shape.hidden:
             svgshape.attrib["style"] += "display: none;"
         return svgshape
@@ -439,11 +510,10 @@ class SvgBuilder(SvgHandler, restructure.AbstractBuilder):
             svgshape = self.build_repeater(shape.lottie, shape.child, shapegroup, out_parent)
         elif isinstance(shape.lottie, objects.RoundedCorners):
             svgshape = self.build_rouded_corners(shape.lottie, shape.child, shapegroup, out_parent)
+        elif isinstance(shape.lottie, objects.Trim):
+            svgshape = self.build_trim_path(shape.lottie, shape.child, shapegroup, out_parent)
         else:
             return self.shapegroup_process_child(shape.child, shapegroup, out_parent)
-        if svgshape:
-            self.set_id(svgshape, shape, force=True)
-            svgshape.attrib["style"] = self.group_to_style(shapegroup)
         return svgshape
 
     def build_repeater(self, shape, child, shapegroup, out_parent):
@@ -458,6 +528,7 @@ class SvgBuilder(SvgHandler, restructure.AbstractBuilder):
         out_parent.remove(original)
 
         g = ElementTree.SubElement(out_parent, "g")
+        self.set_clean_id(g, "repeater")
 
         for copy in range(ncopies-1):
             use = ElementTree.SubElement(g, "use")
@@ -484,31 +555,111 @@ class SvgBuilder(SvgHandler, restructure.AbstractBuilder):
 
     def build_rouded_corners(self, shape, child, shapegroup, out_parent):
         round_amount = shape.radius.get_value(self.time)
-        if isinstance(child, objects.Shape):
-            path = child.to_bezier()
-            bezier = path.shape.get_value(self.time).rounded(round_amount)
-            path.shape.clear_animation(bezier)
-            return self._on_shape(path, shapegroup, out_parent)
+        return self._modifier_process(child, shapegroup, out_parent, self._build_rouded_corners_shape, round_amount)
 
-        if isinstance(child, restructure.RestructuredShapeGroup):
-            self._build_rouded_corners_group(child, round_amount)
-            return self._on_shapegroup(child, out_parent)
+    def _build_rouded_corners_shape(self, shape, round_amount):
+        if not isinstance(shape, objects.Shape):
+            return [shape]
+        path = sh.to_bezier()
+        bezier = path.shape.get_value(self.time).rounded(round_amount)
+        path.shape.clear_animation(bezier)
+        return [path]
 
-        return child
+    def build_trim_path(self, shape, child, shapegroup, out_parent):
+        start = max(0, min(1, shape.start.get_value(self.time) / 100))
+        end = max(0, min(1, shape.end.get_value(self.time) / 100))
+        offset = shape.offset.get_value(self.time) / 360 % 1
 
-    def _build_rouded_corners_group(self, shapegroup, round_amount):
+        multidata = {}
+        length = 0
+
+        if shape.multiple == objects.TrimMultipleShapes.Individually:
+            for visishape in reversed(list(self._modifier_foreach_shape(child))):
+                bez = visishape.to_bezier().shape.get_value(self.time)
+                local_length = bez.rough_length()
+                multidata[visishape] = (bez, length, local_length)
+                length += local_length
+
+        return self._modifier_process(
+            child, shapegroup, out_parent, self._build_trim_path_shape,
+            start+offset, end+offset, multidata, length
+        )
+
+    def _modifier_foreach_shape(self, shape):
+        if isinstance(shape, restructure.RestructuredShapeGroup):
+            for child in shape.children:
+                for chsh in self._modifier_foreach_shape(child):
+                    yield chsh
+        elif isinstance(shape, restructure.RestructuredPathMerger):
+            for p in shape.paths:
+                yield p
+        elif isinstance(shape, objects.Shape):
+            yield shape
+
+    def _modifier_process(self, child, shapegroup, out_parent, callback, *args):
+        children = self._modifier_process_child(child, shapegroup, out_parent, callback, *args)
+        return [self.shapegroup_process_child(ch, shapegroup, out_parent) for ch in children]
+
+    def _trim_offlocal(self, t, local_start, local_length, total_length):
+        gt = (t * total_length - local_start) / local_length
+        return max(0, min(1, gt))
+
+    def _build_trim_path_shape(self, shape, start, end, multidata, total_length):
+        if not isinstance(shape, objects.Shape):
+            return [shape]
+
+        if multidata:
+            bezier, local_start, local_length = multidata[shape]
+            if end > 1:
+                lstart = self._trim_offlocal(start, local_start, local_length, total_length)
+                lend = self._trim_offlocal(end-1, local_start, local_length, total_length)
+                out = []
+                if lstart < 1:
+                    out.append(objects.Path(bezier.segment(lstart, 1)))
+                if lend > 0:
+                    out.append(objects.Path(bezier.segment(0, lend)))
+                return out
+
+            lstart = self._trim_offlocal(start, local_start, local_length, total_length)
+            lend = self._trim_offlocal(end, local_start, local_length, total_length)
+            if lend <= 0 or lstart >= 1:
+                return []
+            if lstart <= 0 and lend >= 1:
+                return [objects.Path(bezier)]
+            seg = bezier.segment(lstart, lend)
+            return [objects.Path(seg)]
+
+        path = shape.to_bezier()
+        bezier = path.shape.get_value(self.time)
+        if end > 1:
+            bez1 = bezier.segment(start, 1)
+            bez2 = bezier.segment(0, end-1)
+            return [objects.Path(bez1), objects.Path(bez2)]
+        else:
+            seg = bezier.segment(start, end)
+            return [objects.Path(seg)]
+
+    def _modifier_process_children(self, shapegroup, out_parent, callback, *args):
         children = []
-        for sh in shapegroup.children:
-            if isinstance(sh, objects.Shape):
-                path = sh.to_bezier()
-                bezier = path.shape.get_value(self.time).rounded(round_amount)
-                path.shape.clear_animation(bezier)
-                sh = path
-            elif isinstance(sh, restructure.RestructuredShapeGroup):
-                self._build_rouded_corners_group(sh, round_amount)
-            children.append(sh)
+        for shape in shapegroup.children:
+            children.extend(self._modifier_process_child(shape, shapegroup, out_parent, callback, *args))
         shapegroup.children = children
 
+    def _modifier_process_child(self, shape, shapegroup, out_parent, callback, *args):
+        if isinstance(shape, restructure.RestructuredShapeGroup):
+            self._modifier_process_children(shape, out_parent, callback, *args)
+            return [shape]
+        elif isinstance(shape, restructure.RestructuredPathMerger):
+            paths = []
+            for p in shape.paths:
+                paths.extend(callback(p, *args))
+            shape.paths = paths
+            if paths:
+                return [shape]
+            return []
+        else:
+            return callback(shape, *args)
+
     def _custom_object_supported(self, shape):
         if has_font and isinstance(shape, font.FontShape):
             return True
