@@ -322,7 +322,7 @@ class SvgParser(SvgHandler):
             dest_trans.skew_axis.value = math.degrees(trans["skew_axis"])
             dest_trans.skew.value = -math.degrees(trans["skew_angle"])
             dest_trans.position.value += trans["translation"]
-            dest_trans.rotation.value += math.degrees(trans["angle"])
+            dest_trans.rotation.value -= math.degrees(trans["angle"])
             dest_trans.scale.value *= trans["scale"]
 
     def parse_style(self, element, parent_style):
@@ -420,6 +420,10 @@ class SvgParser(SvgHandler):
                 opacity = color[3]
             opacity *= float(style.get("fill-opacity", 1))
             fill.opacity.value = opacity * 100
+
+            if style.get("fill-rule", "") == "evenodd":
+                fill.fill_rule = objects.FillRule.EvenOdd
+
             group.add_shape(fill)
 
     def _parseshape_use(self, element, shape_parent, parent_style):
@@ -489,11 +493,11 @@ class SvgParser(SvgHandler):
 
     def _parseshape_rect(self, element, shape_parent, parent_style):
         rect = objects.Rect()
-        w = self._parse_unit(element.attrib["width"])
-        h = self._parse_unit(element.attrib["height"])
+        w = self._parse_unit(element.attrib.get("width", 0))
+        h = self._parse_unit(element.attrib.get("height", 0))
         rect.position.value = NVector(
-            self._parse_unit(element.attrib["x"]) + w / 2,
-            self._parse_unit(element.attrib["y"]) + h / 2
+            self._parse_unit(element.attrib.get("x", 0)) + w / 2,
+            self._parse_unit(element.attrib.get("y", 0)) + h / 2
         )
         rect.size.value = NVector(w, h)
         rx = self._parse_unit(element.attrib.get("rx", 0))
